:: LOGROTATE
:: -----------------------------------------------------------------------------
:: parameters
::      see :param_help
::
@echo off
SETLOCAL
SETLOCAL ENABLEDELAYEDEXPANSION
@SET ABS_PATH_MYSELF=%~dp0
@SET VERSION=0.9.0

:: PARAMETERS
:: -----------------------------------------------------------------------------
@SET PARAM_logfile=.\log_server.txt
@SET PARAM_archive-folder=.\log-archive
@SET PARAM_dry-run=0
@SET /A PARAM_verbosity=0
@SET PARAM_timezone=utc
@SET /A PARAM_keepdays=30


:param_init
@SET tmpPARAM_IN=%1
if "!tmpPARAM_IN!"=="" GOTO param_init_done
@SET tmpPARAM_INIT_AUX=!tmpPARAM_IN!
if "!tmpPARAM_INIT_AUX!"=="-h" (
    GOTO :param_help
) else if "!tmpPARAM_INIT_AUX!"=="--help" (
    GOTO :param_help
) else if "!tmpPARAM_INIT_AUX!"=="-v" (
    echo !VERSION!
    GOTO :eop
) else if "!tmpPARAM_INIT_AUX!"=="--version" (
    echo !VERSION!
    GOTO :eop
) else if "!tmpPARAM_INIT_AUX:~0,2!"=="--" (
    SET tmpPARAM_INIT_NAME=PARAM_!tmpPARAM_INIT_AUX:~2,250!
    SET !tmpPARAM_INIT_NAME!=1
) else if "!tmpPARAM_INIT_AUX:~0,1!"=="-" (
    SET tmpPARAM_INIT_NAME=PARAM_!tmpPARAM_INIT_AUX:~1,250!
) else (
    :: strip enclosing quotes
    SET tmpPARAM_INIT_TMP=%1
    SET tmpPARAM_INIT_TMP=!tmpPARAM_INIT_TMP:"=!
    SET !tmpPARAM_INIT_NAME!=!tmpPARAM_INIT_TMP!
    SET tmpPARAM_INIT_NAME=null
)
shift
GOTO param_init
:param_help
echo. Cliffs of Dover logrotate v-!VERSION!
echo.
echo. USAGE: logrotate.bat [OPTIONS]
echo.
echo. OPTIONS
echo.       --archive-folder ^<path-name^>    archive folder's path and name
echo.                                       default: !PARAM_archive-folder!
echo.
echo.       --logfile ^<path-name^>           logfile's path and name
echo.                                       default: !PARAM_logfile!
echo.
echo.       --dry-run                       Just tell what would be done, don't
echo.                                       actually execute the commands
echo.
echo.       -h, --help                      This help
echo.
echo.       --keepdays ^<0^|1 - ...^>          number of days to keep logs for
echo.                                       0: keep all
echo.                                       default: !PARAM_keepdays!
echo.
echo.       --timezone ^<local^|utc^>          timezone for timestamps
echo.                                       default: !PARAM_timezone!
echo.
echo.       --verbosity ^<0 - 10^>            verbosity of output
echo.                                       default: !PARAM_verbosity!
echo.
echo.       -v,--version                    print version
echo.
echo.
GOTO :eop
:param_init_done


:: PROGRAM
:: -----------------------------------------------------------------------------
@SET CURRENT_DATETIME=
@SET ABS_ARCHIVE=!PARAM_archive-folder!
@SET ABS_LOGFILE=!PARAM_logfile!

:: Do not change datetime format! Purge relies on it!
CALL :getDateTime CURRENT_DATETIME !PARAM_timezone! Y-m-d-His

:: We do not like a trailing backslash
IF !ABS_ARCHIVE:~-1!==\ SET ABS_ARCHIVE=!ABS_ARCHIVE:~0,-1!
CALL :qualifyPath ABS_ARCHIVE !ABS_ARCHIVE!

CALL :qualifyPath ABS_LOGFILE !ABS_LOGFILE!

if !PARAM_dry-run! NEQ 0 (
    echo. *** dry run ***
)
CALL :report 2 "Log file   : !PARAM_logfile!"
CALL :report 2 "             = !ABS_LOGFILE!"
CALL :report 2 "Dest Folder: !PARAM_archive-folder!"
CALL :report 2 "             = !ABS_ARCHIVE!"
CALL :report 2 "DateTime   : !CURRENT_DATETIME!"

:: CHECKS
:: ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:: check if destination folder exists
IF NOT EXIST "!ABS_ARCHIVE!\" (
    CALL :report 0 "No such folder: !ABS_ARCHIVE!"
    GOTO :eop_withError
)

:: MOVE NEW LOGFILE
:: ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
IF NOT EXIST "!ABS_LOGFILE!" (
    CALL :report 1 "No '!PARAM_logfile!' found"
) ELSE (
    @SET TMP_NAME_EXT=
    For %%A in ("!ABS_LOGFILE!") do (
        @SET TMP_NAME_EXT=%%~nxA
    )
    if !PARAM_dry-run! NEQ 0 (
        echo. *** dry run: Would move "!ABS_LOGFILE!"
        echo.              =^> "!ABS_ARCHIVE!\!CURRENT_DATETIME!-!TMP_NAME_EXT!"
    ) ELSE (
        move /y "!ABS_LOGFILE!" "!ABS_ARCHIVE!\!CURRENT_DATETIME!-!TMP_NAME_EXT!"
        CALL :report 1 "* moved !TMP_NAME_EXT!"
    )
)

:: PURGE OLD LOGFILES
:: ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
if 0 NEQ !PARAM_keepdays! (
    @SET /A JULIAN_DAYS_NOW = 0
    @SET /A JULIAN_DAYS_KEEP = 0
    CALL :getJulianDays JULIAN_DAYS_NOW !CURRENT_DATETIME!
    @SET /A JULIAN_DAYS_KEEP = !JULIAN_DAYS_NOW! - !PARAM_keepdays!
    FOR %%A IN ("!ABS_ARCHIVE!\*.*") DO (
        @SET tmp_FILE=%%~nxA
        @SET /A tmp_DAYS=0
        CALL :getJulianDays tmp_DAYS !tmp_FILE!
        if !tmp_DAYS! LSS !JULIAN_DAYS_KEEP! (
            if !PARAM_dry-run! NEQ 0 (
                echo. *** dry run: Would purge !tmp_FILE!
            ) ELSE (
                del /F /Q "!ABS_ARCHIVE!\!tmp_FILE!"
                CALL :report 1 "* purged !tmp_FILE!"
            )
        )
    )
)

:: THE END
:: -----------------------------------------------------------------------------
CALL :report 1 "done"
GOTO :eop

:eop_withError
IF 0 EQU !errorlevel! @SET errorlevel=1
echo.
echo.
echo. *********************************
echo. * ERROR - ERROR - ERROR - ERROR *
echo. *********************************
echo.
echo.

:eop
EXIT /B !errorlevel!
ENDLOCAL


:: FUNCTIONS
:: -----------------------------------------------------------------------------

:: FUNCTION: getDateTime
:: ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:: * copies date time info in variable
::
:: @param %1 name of valiable to set
:: @param %2 "utc" or "local"
:: @param %3 Format (e.g. "Y-m-d H:i:s" results in "2018-11-12 23:03:59")
:: 
::           Y      year, all digits
::           m      month, leading zero
::           d      day,leading zero
::           H      hour, 24 hour format, leading zero
::           i      minute, leading zero
::           s      second, leading zero
::
:getDateTime
@SET getDateTime_RSLT=1
@SET getDateTime_VAR=%1
@SET getDateTime_TZ=%2
@SET getDateTime_FORMAT=%3
@SET getDateTime_ECHO=%4
@SET getDateTime_VAL=

:: remove quotes
@SET getDateTime_VAR=%getDateTime_VAR:"=%
@SET getDateTime_TZ=%getDateTime_TZ:"=%
@SET getDateTime_FORMAT=%getDateTime_FORMAT:"=%

@IF "utc"=="%getDateTime_TZ%" (
    @for /f %%x in ('wmic path win32_utctime get /format:list ^| findstr "="') do @SET getDateTime_%%x
) ELSE (    
    @for /f %%x in ('wmic path win32_localtime get /format:list ^| findstr "="') do @SET getDateTime_%%x
)

@IF %getDateTime_Month% LEQ 9 SET getDateTime_Month=0%getDateTime_Month%
@IF %getDateTime_Day% LEQ 9 SET getDateTime_Day=0%getDateTime_Day%
@IF %getDateTime_Hour% LEQ 9 SET getDateTime_Hour=0%getDateTime_Hour%
@IF %getDateTime_Minute% LEQ 9 SET getDateTime_Minute=0%getDateTime_Minute%
@IF %getDateTime_Second% LEQ 9 SET getDateTime_Second=0%getDateTime_Second%

@REM echo getDateTime_Day         : %getDateTime_Day%
@REM echo getDateTime_DayOfWeek   : %getDateTime_DayOfWeek%
@REM echo getDateTime_Hour        : %getDateTime_Hour%
@REM echo getDateTime_Milliseconds: %getDateTime_Milliseconds%
@REM echo getDateTime_Minute      : %getDateTime_Minute%
@REM echo getDateTime_Month       : %getDateTime_Month%
@REM echo getDateTime_Quarter     : %getDateTime_Quarter%
@REM echo getDateTime_Second      : %getDateTime_Second%
@REM echo getDateTime_WeekInMonth : %getDateTime_WeekInMonth%
@REM echo getDateTime_Year        : %getDateTime_Year%

@SET getDateTime_FORMAT=%getDateTime_FORMAT%___ENDOFSTRING___
:getDateTime_loop1
    :: get first character from input
    @SET getDateTime_FORMAT_chr=%getDateTime_FORMAT:~0,1%
    :: do something with %char%, e.g. simply print it out
    @REM ECHO char: %getDateTime_FORMAT_chr%
    @if d EQU %getDateTime_FORMAT_chr% ( SET getDateTime_VAL=%getDateTime_VAL%%getDateTime_Day%
    ) else if m EQU %getDateTime_FORMAT_chr% ( SET getDateTime_VAL=%getDateTime_VAL%%getDateTime_Month%
    ) else if Y EQU %getDateTime_FORMAT_chr% ( SET getDateTime_VAL=%getDateTime_VAL%%getDateTime_Year%
    ) else if H EQU %getDateTime_FORMAT_chr% ( SET getDateTime_VAL=%getDateTime_VAL%%getDateTime_Hour%
    ) else if i EQU %getDateTime_FORMAT_chr% ( SET getDateTime_VAL=%getDateTime_VAL%%getDateTime_Minute%
    ) else if s EQU %getDateTime_FORMAT_chr% ( SET getDateTime_VAL=%getDateTime_VAL%%getDateTime_Second%
    ) else SET getDateTime_VAL=%getDateTime_VAL%%getDateTime_FORMAT_chr%
    
    :: remove first character from input
    @SET getDateTime_FORMAT=%getDateTime_FORMAT:~1%
    :: repeat until only the string terminator is left
    @IF NOT "%getDateTime_FORMAT%" == "___ENDOFSTRING___" @GOTO getDateTime_loop1


@SET getDateTime_RSLT=0
@SET %getDateTime_VAR%=%getDateTime_VAL%
:getDateTime_eop
@EXIT /B !getDateTime_RSLT!

:: FUNCTION getJulianDays
:: ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:: @param %1 Name of receiving var
:: @param %2 Input date YYYY-MM-DD
:getJulianDays
@SET getJulianDays_VAR=%1
@SET getJulianDays_DATE=%~2
@SET getJulianDays_YY=
@SET getJulianDays_MM=
@SET getJulianDays_DD=
@SET getJulianDays_NUM=
FOR /F "TOKENS=1-3 DELIMS=-" %%A IN ("!getJulianDays_DATE!") DO SET getJulianDays_YY=%%A& SET getJulianDays_MM=%%B& SET getJulianDays_DD=%%C
REM ELIMINATE LEFT ZEROS
SET /A getJulianDays_DD=10%getJulianDays_DD% %% 100
SET /A getJulianDays_MM=10%getJulianDays_MM% %% 100
REM CALCULATE JULIAN DAY NUMBER
IF %getJulianDays_MM% LSS 3 (
    SET /A getJulianDays_MM = !getJulianDays_MM! + 12
    SET /A getJulianDays_YY = !getJulianDays_YY! - 1
)
SET /A getJulianDays_A = !getJulianDays_YY! / 100
SET /A getJulianDays_B = !getJulianDays_A! / 4
SET /A getJulianDays_C = 2 - !getJulianDays_A! + !getJulianDays_B!
SET /A getJulianDays_D = 36525 * (!getJulianDays_YY! + 4716) / 100
SET /A getJulianDays_E = 306 * (!getJulianDays_MM! +1 ) / 10
SET /A getJulianDays_NUM = !getJulianDays_C! + !getJulianDays_DD! + !getJulianDays_D! + !getJulianDays_E! - 1524
@SET !getJulianDays_VAR!=!getJulianDays_NUM!
EXIT /B 0

:: FUNCTION: qualifyPath
:: ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:: Changes "C:\dir1\..\dir2\" to "C:\dir2\"
:: @param %1 name of var
:: @param %2 path
:: @return 0
:qualifyPath
@SET qualifyPath_VAR=%1
@SET !qualifyPath_VAR!=%~f2
EXIT /B 0

:: FUNCTION: report
:: ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:: @param %1 level
:: @param %2 text
:: @return 0
:report
@SET report_LEVEL=%1
@SET report_TEXT=%~2
if !report_LEVEL! LEQ !PARAM_verbosity! (
    echo. !report_TEXT!
)
EXIT /B 0
